@@ -1,4 +1,4 @@
-__version__ = "1.9.0"
+__version__ = "1.9.4"
 
 
 from .hvtools import beta_plots, hist
@@ -62,6 +62,8 @@ from .stream_plotter import StreamPlotter
 from .tracker import Tracker
 from .filtering.tvd import tvd
 from .vonmises import VonMisesFitter
+from .ez_agent import EZAgent
+from .agent_runner import AgentRunner
 
 from .dataframe_tools import get_quick_schema_class, get_pandas_sql_class
 from .plotting import ColorCyle
@@ -76,6 +78,7 @@ warnings_mute = mute_warnings
 
 # Export list
 __all__ = [
+    "AgentRunner",
     "Bernstein",
     "BernsteinFitter",
     "BlobAttr",
@@ -87,6 +90,7 @@ __all__ = [
     "DuckMirror",
     "duckloader_factory",
     "Elliptic",
+    "EZAgent",
     "Fitter",
     "Gemini",
     "GlobalClock",
